@@ -1,3 +1,3 @@
 from .render_out_mock import RenderOutMock
-from .subproc_vec_env_lock import SubprocVecEnvLock
-from .robots import RobotJiminyEnv, RobotJiminyGoalEnv
\ No newline at end of file
+from .robots import RobotJiminyEnv, RobotJiminyGoalEnv
+from .flatten_observation import FlattenObservation
